#ifndef CTIMETAG_ONCE
#define CTIMETAG_ONCE

#define _CRT_SECURE_NO_DEPRECATE  //supress microsoft warnings
#include <string>  
//#include <stdio.h>


typedef long long TimeType;
typedef unsigned char ChannelType;



namespace TimeTag
{

class CUsb;
class CHelper;
class CTimetagReader;
class CLogic;
class CTest;
class CGiga;
class CLock;
class CompactFileConverter;
			

		class Exception 
	{
	private:
		std::string text;
		
	public:
		Exception(const char * m):
			text(m){}  
	Exception(std::string m):
			text(m)
		{
		}

	std::string GetMessageText() const
		{
			return text;
		}

	};
			
		struct Result
		{
			TimeType time;
			ChannelType channel;
		};
		//class CompactFileConverter;
class CTimeTag
{
private:
   		CUsb *usb;
		CHelper *helper;
		CTimetagReader *reader;
		CLogic *logic;
		CTest *test;
		CGiga * giga;
		CLock * lock;
		
		double caldata[64];
		double resolution;
		bool isGiga;
		CompactFileConverter *c = nullptr;
		//CompactFileConverter *c;

		//For Python interfafce only
		ChannelType* channel_data;
		TimeType *time_data;
		int size;

		
		 
public:

		CTimeTag();
		~CTimeTag();

		CTimetagReader * GetReader();
		CLogic * GetLogic();

		void PythonInternReadData()
		{
			size = ReadTags(channel_data, time_data);
		}
		void PythonInternGetChannels(unsigned char** channel_ret, int *channel_size)
		{
			*channel_size = size;
			*channel_ret = channel_data;
		}

		void PythonInternGetTimes(long long**time_ret, int*time_size)
		{
			*time_size = size;
			*time_ret = time_data;

		}

		void Open(int nr= 1);
		void OpenUSB(int nr= 1);
		void OpenPCIe(int nr= 1);
		bool IsOpen();
		void Close();
		void Calibrate();
        void SetInputThreshold(int input, double voltage);
        void SetFilterMinCount(int MinCount);

        void SetFilterMaxTime(int MaxTime);
		 
        double GetResolution()  ;
        int GetFpgaVersion()  ;
        void SetLedBrightness(int percent) ;
		std::string GetErrorText(long long flags);
        void EnableGating(bool enable);
        void GatingLevelMode(bool enable);
        void SetGateWidth(int duration);
		void SwitchSoftwareGate(bool onOff);
		 void StartBufferedSaving (char *filename, int bufferSizeBytes);


        void SetInversionMask(long long mask);
        void SetDelay(int Input, int Delay);
        long long ReadErrorFlags();
        int GetNoInputs();
        void UseTimetagGate(bool use);
        void UseLevelGate(bool p);
        bool LevelGateActive();
        void Use10MHz(bool use);  
		void UseCompactFormat(bool use);
		
        void SetFilterException(int exception);
		CTest * GetTest();
		CHelper * GetHelper();

		void StartTimetags();
		void StartStreaming();
		void StopStreaming();
		void StopTimetags();
		int ReadTags(ChannelType*& channel_ret, TimeType *&time_ret);
		bool ReadNextTag(ChannelType & channel, TimeType &time);
		struct Result* ReadNextTagValue();
		void StartConvertCompactFile(char * inFileName, char * outFileName);
		void SaveDcCalibration(char * filename);
		void LoadDcCalibration(char * filename);
		void SetFG(int periode, int high);
		int GetSingleCount(int i);
		double FreezeSingleCounter();
		char * ConvertCompactFileStatus();
		void AbortConvertCompactFile();

		bool ConvertCompactFileRunning();
		int ActivateTestSignal(long long  mask);

private:
//		void CheckOpen();
		void Init();
		void OpenSub(int nr, bool PCIe);
		double GetDcCalibration(int chan);


};
}

#endif
